% Mine example demonstration for Optimal Switching paper
mu=0.01;
sigma=0.02;
rho=0.04;
h=1;
k=2;
C12=5;
C21=2;

clear model
model.func='minemodel';
model.params={mu,sigma,rho,h,k};
model.C=[0 C12;C21 0];
% the next line is not needed but illustrates how to set the value function
% to specific values
% model.values=[nan 0 1 0;nan 0 2 -C21];  


% solve as a two dimensional problem
fspace=fundefn('lin',[51 51],[0 0],[100 10]);
[cv,snodes,x]=ossolve(model,fspace);
z=getboundaries(snodes,x);
z12=z{1,2};
z21=z{2,1};
z12(:,2)=-z12(:,2); z12=sortrows(z12,[1 2]);z12(:,2)=-z12(:,2);
z21(:,2)=-z21(:,2); z21=sortrows(z21,[1 2]);z21(:,2)=-z21(:,2);

figure(1); 
plot(z12(:,1),z12(:,2),'-r',z21(:,1),z21(:,2),'-k')
title('Approximate Regime Switching Boundaries')
xlabel('Q')
ylabel('P')

% Solve using one-dimensional simplification
% the next line is not needed but illustrates how to set the value function
% to specific values
% model.values=[0 1 0;0 2 -C21];

fspace2=fundefn('lin',[501],[0],[50]);
[cv2,snodes,x,xindex]=ossolve(model,fspace2);

% Obtain the nearly closed form solution
beta=roots([sigma^2/2 mu-h-sigma^2/2 -rho]); beta2=beta(beta<0);
beta=roots([sigma^2/2 mu-sigma^2/2 -rho]);   beta1=beta(beta>0);
sstar=broyden('mineres',[xindex(1,4);xindex(2,1)],beta1,beta2,mu,sigma,rho,h,k,C12,C21);
[res,A]=mineres(sstar,beta1,beta2,mu,sigma,rho,h,k,C12,C21);


% Plot nearly closed form 
xx=linspace(1e-5,50,501)'; 
%hold on; plot(xx,xindex(1,4)./xx,'--r'); plot(xx,xindex(2,1)./xx,'--k'); hold off
hold on; plot(xx,sstar(1)./xx,'--r'); plot(xx,sstar(2)./xx,'--k'); hold off

xlim([0 50])
ylim([0 5])

y=linspace(0,50,501)';
figure(2)
V1=A(1)*y.^beta1;
V2=A(2)*y.^beta2+h*y/(rho-mu+h)-k/rho;
V1(y>sstar(1))=V2(y>sstar(1))-C12;
V2(y<sstar(2))=V1(y<sstar(2))-C21;
plot(y,[funeval(cv2,fspace2,y)-[V1 V2]])
title('Value Function Approximation Error')
xlabel('y')


figure(3)
regimeplot(xindex,{'Inactive','Active'},0,50)
title('Mine Problem Regime Plot')
xlabel('y')

figure(1)

% Compare 1-D OSSOLVE solution to nearly closed form solution
disp('OSSOLVE and Actual Optimal Switching Locations')
disp([xindex(1,4) xindex(2,1);sstar'])
