% OSOPTIMUM Evaluates the optimal decision strategy for optimal switching models
% USAGE
%   [regime,V,dV]=osoptimum(S,R,cv,model,fspace);
% INPUTS
%   S : an nxd matrix of continuous state values
%   R : an nx1 vector of regime values (or a scalar value)
%   cv : coefficients of value function approximation
%   fspace : function approximation structure variable
% OUTPUTS
%   regime : the optimal regime choices for (S,R)
%   V      : the value function at (S,R)
%   dV     : the marginal value (shadow price) function at (S,R)
function [regime,V,dV]=osoptimum(S,R,cv,model,fspace)
tol = optget('osoptimum','tol',1e-5);
n=size(S,1);
if length(R)==1, R=R(ones(n,1)); end
V=funeval(cv,fspace,S);
V0=V((1:n)'+(R-1)*n);       % value function if no switch occurs 

C=model.cost;
V1=V-C(R,:);                % value function if switch occurs
V1((1:n)'+(R-1)*n)=-inf;
[V,regime]=max(V1,[],2);    % best switch   
ind=V0-V>tol.*abs(V0);      % check if no-switch gives enough improvement
V(ind)=V0(ind);
regime(ind)=R(ind);

if nargout>2, dV=funeval(cv,fspace,S,eye(size(S,2))); dV=dV((1:n)'+(R-1)*n); end