% REGIMEPLOT Creates a plot of boundary behavior for 1-D optimal switching models
% USAGE
%   regimeplot(xindex,regimenames,a,b)
% INPUTS
%   xindex       : fourth output from dscsolve
%   regimesnames : a cell array of strings used for y-axis tickmarks [optional]
%   a            : lower bound for x-axis [optional]
%   b            : upper bound for x axis [optional]

function regimeplot(xindex,regimenames,a,b)
m=size(xindex,1);

if nargin<2 | isempty(regimenames)
  regimenames=cell(1,m);
  for i=1:m, regimenames{i}=num2str(i); end
end
if nargin<3 | isempty(a), a=min(xindex(:,1)); end
if nargin<4 | isempty(b), b=max(xindex(:,4)); end

lower=xindex(xindex(:,2)>0,[1 2]);
upper=xindex(xindex(:,5)>0,[4 5]);

color='k';
fontsize=24;


h1=plot([a;b]*ones(1,m),[1;1]*(1:m),'k-');
xlim([a b])
hold on
set(gca,'YTickLabel',fliplr(regimenames),'YTick',1:m)
for i=1:m
  if xindex(i,2)>0
    if xindex(i,5)>0
       h=plot([xindex(i,1) xindex(i,1) xindex(i,4) xindex(i,4)],...
         [m+1-xindex(i,2) m+1-i m+1-i m+1-xindex(i,5)],'k-');  
       if i>xindex(i,2)
         text(xindex(i,1),m+1-xindex(i,2),'\uparrow','VerticalAlignment','top','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       else
         text(xindex(i,1),m+1-xindex(i,2),'\downarrow','VerticalAlignment','baseline','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       end
       if i>xindex(i,5)
         text(xindex(i,4),m+1-xindex(i,5),'\uparrow','VerticalAlignment','top','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       else
         text(xindex(i,4),m+1-xindex(i,5),'\downarrow','VerticalAlignment','baseline','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       end
    else
       h=plot([xindex(i,1) xindex(i,1) xindex(i,4)],...
         [m+1-xindex(i,2) m+1-i m+1-i],'k-');
       if i>xindex(i,2)
         text(xindex(i,1),m+1-xindex(i,2),'\uparrow','VerticalAlignment','top','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       else
         text(xindex(i,1),m+1-xindex(i,2),'\downarrow','VerticalAlignment','baseline','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       end
    end
  elseif xindex(i,5)>0
    h=plot([xindex(i,1) xindex(i,4) xindex(i,4)],[m+1-i m+1-i m+1-xindex(i,5)],'k-');
       if i>xindex(i,5)
         text(xindex(i,4),m+1-xindex(i,5),'\uparrow','VerticalAlignment','top','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       else
         text(xindex(i,4),m+1-xindex(i,5),'\downarrow','VerticalAlignment','baseline','HorizontalAlignment','center','FontSize',fontsize,'Color',color)
       end
  else
    h=plot([xindex(i,1) xindex(i,4)],[m+1-i m+1-i],'k-');
  end
  set(h,'Linewidth',3,'Color',color)
end
hold off
ylim([0 m+1])

